@@ -230,7 +230,7 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
 		gdata_parser_string_append_escaped (xml_string, " label='", priv->label, "'");
 
 	if (priv->relation_type != NULL)
-		g_string_append_printf (xml_string, " rel='%s'", priv->relation_type);
+		gdata_parser_string_append_escaped (xml_string, " rel='", priv->relation_type, "'");
 
 	if (priv->value_string != NULL)
 		gdata_parser_string_append_escaped (xml_string, " valueString='", priv->value_string, "'");
