@@ -91,7 +91,7 @@ gdata_buffer_free (GDataBuffer *self)
  * @data: the data to push onto the buffer
  * @length: the length of @data
  *
- * Pushes @length bytes of @data onto the buffer, taking a copy of the data. If @data is %NULL and @length is %0,
+ * Pushes @length bytes of @data onto the buffer, taking a copy of the data. If @data is %NULL and @length is <code class="literal">0</code>,
  * the buffer will be marked as having reached the EOF, and subsequent calls to gdata_buffer_push_data()
  * will fail and return %FALSE.
  *
@@ -131,7 +131,8 @@ gdata_buffer_push_data (GDataBuffer *self, const guint8 *data, gsize length)
 	chunk->next = NULL;
 
 	/* Copy the data to the chunk */
-	memcpy (chunk->data, data, length);
+	if (G_LIKELY (data != NULL))
+		memcpy (chunk->data, data, length);
 
 	/* Add it to the buffer's tail */
 	if (self->tail != NULL)
@@ -223,7 +224,7 @@ gdata_buffer_pop_data (GDataBuffer *self, guint8 *data, gsize length_requested,
 			cancelled_data.buffer = self;
 			cancelled_data.cancelled = &cancelled;
 
-			cancelled_signal = g_signal_connect (cancellable, "cancelled", (GCallback) pop_cancelled_cb, &cancelled_data);
+			cancelled_signal = g_cancellable_connect (cancellable, (GCallback) pop_cancelled_cb, &cancelled_data, NULL);
 		}
 
 		/* Block until more data is available */
@@ -242,7 +243,7 @@ gdata_buffer_pop_data (GDataBuffer *self, guint8 *data, gsize length_requested,
 
 		/* Disconnect from the cancelled signal */
 		if (cancellable != NULL)
-			g_signal_handler_disconnect (cancellable, cancelled_signal);
+			g_cancellable_disconnect (cancellable, cancelled_signal);
 	} else {
 		return_length = length_requested;
 	}
@@ -305,9 +306,9 @@ gdata_buffer_pop_data (GDataBuffer *self, guint8 *data, gsize length_requested,
  * Pops as much data as possible off the #GDataBuffer, up to a limit of @maxium_length bytes. If fewer bytes exist
  * in the buffer, fewer bytes will be returned. If more bytes exist in the buffer, @maximum_length bytes will be returned.
  *
- * If %0 bytes exist in the buffer, this function will block until data is available. Otherwise, it will never block.
+ * If <code class="literal">0</code> bytes exist in the buffer, this function will block until data is available. Otherwise, it will never block.
  *
- * Return value: the number of bytes returned in @data (guaranteed to be more than %0 and at most @maximum_length)
+ * Return value: the number of bytes returned in @data (guaranteed to be more than <code class="literal">0</code> and at most @maximum_length)
  *
  * Since: 0.5.0
  **/
