@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2008–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -58,10 +58,11 @@ struct _GDataFeedPrivate {
 	gchar *id;
 	gchar *etag;
 	GTimeVal updated;
-	GList *categories;
+	GList *categories; /* GDataCategory */
 	gchar *logo;
-	GList *links;
-	GList *authors;
+	gchar *icon;
+	GList *links; /* GDataLink */
+	GList *authors; /* GDataAuthor */
 	GDataGenerator *generator;
 	guint items_per_page;
 	guint start_index;
@@ -75,6 +76,7 @@ enum {
 	PROP_TITLE,
 	PROP_SUBTITLE,
 	PROP_LOGO,
+	PROP_ICON,
 	PROP_GENERATOR,
 	PROP_ITEMS_PER_PAGE,
 	PROP_START_INDEX,
@@ -99,6 +101,7 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	parsable_class->pre_parse_xml = pre_parse_xml;
 	parsable_class->parse_xml = parse_xml;
 	parsable_class->post_parse_xml = post_parse_xml;
+	parsable_class->element_name = "feed";
 
 	/**
 	 * GDataFeed:title:
@@ -154,7 +157,6 @@ gdata_feed_class_init (GDataFeedClass *klass)
 					NULL,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-
 	/**
 	 * GDataFeed:updated:
 	 *
@@ -182,6 +184,21 @@ gdata_feed_class_init (GDataFeedClass *klass)
 					NULL,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GDataFeed:icon:
+	 *
+	 * The URI of an icon for the feed.
+	 *
+	 * API reference: <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.icon">atom:icon</ulink>
+	 *
+	 * Since: 0.6.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_ICON,
+					 g_param_spec_string ("icon",
+							      "Icon", "The URI of an icon for the feed.",
+							      NULL,
+							      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * GDataFeed:generator:
 	 *
@@ -191,8 +208,9 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * atom:generator</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_GENERATOR,
-				g_param_spec_pointer ("generator",
+				g_param_spec_object ("generator",
 					"Generator", "Details of the software used to generate the feed.",
+					GDATA_TYPE_GENERATOR,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -253,7 +271,7 @@ gdata_feed_init (GDataFeed *self)
 static void
 gdata_feed_dispose (GObject *object)
 {
-	GDataFeedPrivate *priv = GDATA_FEED_GET_PRIVATE (object);
+	GDataFeedPrivate *priv = GDATA_FEED (object)->priv;
 
 	if (priv->entries != NULL) {
 		g_list_foreach (priv->entries, (GFunc) g_object_unref, NULL);
@@ -261,6 +279,28 @@ gdata_feed_dispose (GObject *object)
 	}
 	priv->entries = NULL;
 
+	if (priv->categories != NULL) {
+		g_list_foreach (priv->categories, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->categories);
+	}
+	priv->categories = NULL;
+
+	if (priv->links != NULL) {
+		g_list_foreach (priv->links, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->links);
+	}
+	priv->links = NULL;
+
+	if (priv->authors != NULL) {
+		g_list_foreach (priv->authors, (GFunc) g_object_unref, NULL);
+		g_list_free (priv->authors);
+	}
+	priv->authors = NULL;
+
+	if (priv->generator != NULL)
+		g_object_unref (priv->generator);
+	priv->generator = NULL;
+
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_feed_parent_class)->dispose (object);
 }
@@ -268,20 +308,14 @@ gdata_feed_dispose (GObject *object)
 static void
 gdata_feed_finalize (GObject *object)
 {
-	GDataFeedPrivate *priv = GDATA_FEED_GET_PRIVATE (object);
+	GDataFeedPrivate *priv = GDATA_FEED (object)->priv;
 
-	xmlFree (priv->title);
-	xmlFree (priv->subtitle);
-	xmlFree (priv->id);
-	xmlFree (priv->etag);
-	g_list_foreach (priv->categories, (GFunc) gdata_category_free, NULL);
-	g_list_free (priv->categories);
-	xmlFree (priv->logo);
-	g_list_foreach (priv->links, (GFunc) gdata_link_free, NULL);
-	g_list_free (priv->links);
-	g_list_foreach (priv->authors, (GFunc) gdata_author_free, NULL);
-	g_list_free (priv->authors);
-	gdata_generator_free (priv->generator);
+	g_free (priv->title);
+	g_free (priv->subtitle);
+	g_free (priv->id);
+	g_free (priv->etag);
+	g_free (priv->logo);
+	g_free (priv->icon);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_feed_parent_class)->finalize (object);
@@ -290,7 +324,7 @@ gdata_feed_finalize (GObject *object)
 static void
 gdata_feed_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
 {
-	GDataFeedPrivate *priv = GDATA_FEED_GET_PRIVATE (object);
+	GDataFeedPrivate *priv = GDATA_FEED (object)->priv;
 
 	switch (property_id) {
 		case PROP_TITLE:
@@ -311,8 +345,11 @@ gdata_feed_get_property (GObject *object, guint property_id, GValue *value, GPar
 		case PROP_LOGO:
 			g_value_set_string (value, priv->logo);
 			break;
+		case PROP_ICON:
+			g_value_set_string (value, priv->icon);
+			break;
 		case PROP_GENERATOR:
-			g_value_set_pointer (value, priv->generator);
+			g_value_set_object (value, priv->generator);
 			break;
 		case PROP_ITEMS_PER_PAGE:
 			g_value_set_uint (value, priv->items_per_page);
@@ -358,232 +395,164 @@ typedef struct {
 	guint total_results;
 } ProgressCallbackData;
 
-static gboolean
-progress_callback_idle (ProgressCallbackData *data)
-{
-	data->progress_callback (data->entry, data->entry_i, data->total_results, data->progress_user_data);
-	g_object_unref (data->entry);
-	g_slice_free (ProgressCallbackData, data);
-	return FALSE;
-}
-
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GDataFeed *self;
+	GDataFeed *self = GDATA_FEED (parsable);
 	ParseData *data = user_data;
 
-	g_return_val_if_fail (GDATA_IS_FEED (parsable), FALSE);
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (node != NULL, FALSE);
+	if (gdata_parser_is_namespace (node, "http://www.w3.org/2005/Atom") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "entry") == 0) {
+			/* atom:entry */
+			GDataEntry *entry;
+			GType entry_type;
 
-	self = GDATA_FEED (parsable);
+			/* Allow @data to be %NULL, and assume we're parsing a vanilla feed, so that we can test #GDataFeed in tests/general.c. A little hacky,
+			 * but not too much so, and valuable for testing. */
+			entry_type = (data != NULL) ? data->entry_type : GDATA_TYPE_ENTRY;
+			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (entry_type, doc, node, NULL, error));
+			if (entry == NULL)
+				return FALSE;
 
-	if (xmlStrcmp (node->name, (xmlChar*) "entry") == 0) {
-		/* atom:entry */
-		GDataEntry *entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (data->entry_type, "entry", doc, node, NULL, error));
-		if (entry == NULL)
-			return FALSE;
+			/* Calls the callbacks in the main thread */
+			if (data != NULL)
+				_gdata_feed_call_progress_callback (self, data, entry);
+			_gdata_feed_add_entry (self, entry);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
+			/* atom:title */
+			if (self->priv->title != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		/* Call the progress callback in the main thread */
-		if (data->progress_callback != NULL) {
-			ProgressCallbackData *progress_data;
+			self->priv->title = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "subtitle") == 0) {
+			/* atom:subtitle */
+			if (self->priv->subtitle != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
 
-			/* Build the data for the callback */
-			progress_data = g_slice_new (ProgressCallbackData);
-			progress_data->progress_callback = data->progress_callback;
-			progress_data->progress_user_data = data->progress_user_data;
-			progress_data->entry = g_object_ref (entry);
-			progress_data->entry_i = data->entry_i;
-			progress_data->total_results = MIN (self->priv->items_per_page, self->priv->total_results);
+			self->priv->subtitle = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "id") == 0 && xmlStrcmp (node->ns->href, (xmlChar*) "http://www.w3.org/2005/Atom") == 0) {
+			/* atom:id */
+			/* The namespace check is necessary because there's an "id" element in the gphoto namespace (PicasaWeb service) */
+			if (self->priv->id != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
 
-			/* Send the callback; use G_PRIORITY_DEFAULT rather than G_PRIORITY_DEFAULT_IDLE
-			 * to contend with the priorities used by the callback functions in GAsyncResult */
-			g_idle_add_full (G_PRIORITY_DEFAULT, (GSourceFunc) progress_callback_idle, progress_data, NULL);
-		}
+			self->priv->id = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "updated") == 0) {
+			/* atom:updated */
+			xmlChar *updated_string;
 
-		data->entry_i++;
-		self->priv->entries = g_list_prepend (self->priv->entries, entry);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
-		/* atom:title */
-		if (self->priv->title != NULL)
-			return gdata_parser_error_duplicate_element ("title", "feed", error);
+			/* Duplicate checking */
+			if (self->priv->updated.tv_sec != 0 || self->priv->updated.tv_usec != 0)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		self->priv->title = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "subtitle") == 0) {
-		/* atom:subtitle */
-		if (self->priv->subtitle != NULL)
-			return gdata_parser_error_duplicate_element ("subtitle", "feed", error);
-
-		self->priv->subtitle = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "id") == 0) {
-		/* atom:id */
-		if (self->priv->id != NULL)
-			return gdata_parser_error_duplicate_element ("id", "feed", error);
-
-		self->priv->id = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "updated") == 0) {
-		/* atom:updated */
-		xmlChar *updated_string;
-
-		/* Duplicate checking */
-		if (self->priv->updated.tv_sec != 0 || self->priv->updated.tv_usec != 0)
-			return gdata_parser_error_duplicate_element ("updated", "feed", error);
-
-		/* Parse the string */
-		updated_string = xmlNodeListGetString (doc, node->children, TRUE);
-		if (g_time_val_from_iso8601 ((gchar*) updated_string, &(self->priv->updated)) == FALSE) {
-			gdata_parser_error_not_iso8601_format ("updated", "feed", (gchar*) updated_string, error);
-			xmlFree (updated_string);
-			return FALSE;
-		}
-
-		xmlFree (updated_string);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
-		/* atom:category */
-		xmlChar *scheme, *term, *label;
-		GDataCategory *category;
-
-		scheme = xmlGetProp (node, (xmlChar*) "scheme");
-		term = xmlGetProp (node, (xmlChar*) "term");
-		label = xmlGetProp (node, (xmlChar*) "label");
-
-		category = gdata_category_new ((gchar*) term, (gchar*) scheme, (gchar*) label);
-		self->priv->categories = g_list_prepend (self->priv->categories, category);
-
-		xmlFree (scheme);
-		xmlFree (term);
-		xmlFree (label);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "logo") == 0) {
-		/* atom:logo */
-		if (self->priv->logo != NULL)
-			return gdata_parser_error_duplicate_element ("logo", "feed", error);
-
-		self->priv->logo = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
-		/* atom:link */
-		xmlChar *href, *rel, *type, *hreflang, *link_title, *link_length;
-		gint length_int;
-		GDataLink *link;
-
-		href = xmlGetProp (node, (xmlChar*) "href");
-		rel = xmlGetProp (node, (xmlChar*) "rel");
-		type = xmlGetProp (node, (xmlChar*) "type");
-		hreflang = xmlGetProp (node, (xmlChar*) "hreflang");
-		link_title = xmlGetProp (node, (xmlChar*) "title");
-		link_length = xmlGetProp (node, (xmlChar*) "length");
-
-		if (link_length == NULL)
-			length_int = -1;
-		else
-			length_int = strtoul ((gchar*) link_length, NULL, 10);
-
-		link = gdata_link_new ((gchar*) href, (gchar*) rel, (gchar*) type, (gchar*) hreflang, (gchar*) link_title, length_int);
-		self->priv->links = g_list_prepend (self->priv->links, link);
-
-		xmlFree (href);
-		xmlFree (rel);
-		xmlFree (type);
-		xmlFree (hreflang);
-		xmlFree (link_title);
-		xmlFree (link_length);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "author") == 0) {
-		/* atom:author */
-		GDataAuthor *author;
-		xmlNode *author_node;
-		xmlChar *name = NULL, *uri = NULL, *email = NULL;
-
-		author_node = node->children;
-		while (author_node != NULL) {
-			if (xmlStrcmp (author_node->name, (xmlChar*) "name") == 0) {
-				name = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else if (xmlStrcmp (author_node->name, (xmlChar*) "uri") == 0) {
-				uri = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else if (xmlStrcmp (author_node->name, (xmlChar*) "email") == 0) {
-				email = xmlNodeListGetString (doc, author_node->children, TRUE);
-			} else {
-				gdata_parser_error_unhandled_element ((gchar*) author_node->ns->prefix, (gchar*) author_node->name, "author", error);
-				xmlFree (name);
-				xmlFree (uri);
-				xmlFree (email);
+			/* Parse the string */
+			updated_string = xmlNodeListGetString (doc, node->children, TRUE);
+			if (g_time_val_from_iso8601 ((gchar*) updated_string, &(self->priv->updated)) == FALSE) {
+				gdata_parser_error_not_iso8601_format (node, (gchar*) updated_string, error);
+				xmlFree (updated_string);
 				return FALSE;
 			}
+			xmlFree (updated_string);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
+			/* atom:category */
+			GDataCategory *category = GDATA_CATEGORY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_CATEGORY, doc, node, NULL, error));
+			if (category == NULL)
+				return FALSE;
 
-			author_node = author_node->next;
+			self->priv->categories = g_list_prepend (self->priv->categories, category);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "logo") == 0) {
+			/* atom:logo */
+			if (self->priv->logo != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
+
+			self->priv->logo = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "icon") == 0) {
+			/* atom:icon */
+			if (self->priv->icon != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
+
+			self->priv->icon = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
+			/* atom:link */
+			GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
+			if (link == NULL)
+				return FALSE;
+
+			self->priv->links = g_list_prepend (self->priv->links, link);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "author") == 0) {
+			/* atom:author */
+			GDataAuthor *author = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
+			if (author == NULL)
+				return FALSE;
+
+			self->priv->authors = g_list_prepend (self->priv->authors, author);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "generator") == 0) {
+			/* atom:generator */
+			GDataGenerator *generator;
+
+			/* Duplicate checking */
+			if (self->priv->generator != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
+
+			generator = GDATA_GENERATOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GENERATOR, doc, node, NULL, error));
+			if (generator == NULL)
+				return FALSE;
+
+			self->priv->generator = generator;
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_feed_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
+	} else if (gdata_parser_is_namespace (node, "http://a9.com/-/spec/opensearch/1.1/") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "totalResults") == 0) {
+			/* openSearch:totalResults */
+			xmlChar *total_results_string;
 
-		author = gdata_author_new ((gchar*) name, (gchar*) uri, (gchar*) email);
-		self->priv->authors = g_list_prepend (self->priv->authors, author);
+			/* Duplicate checking */
+			if (self->priv->total_results != 0)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		xmlFree (name);
-		xmlFree (uri);
-		xmlFree (email);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "generator") == 0) {
-		/* atom:generator */
-		xmlChar *name, *uri, *version;
+			/* Parse the number */
+			total_results_string = xmlNodeListGetString (doc, node->children, TRUE);
+			if (total_results_string == NULL)
+				return gdata_parser_error_required_content_missing (node, error);
 
-		/* Duplicate checking */
-		if (self->priv->generator != NULL)
-			return gdata_parser_error_duplicate_element ("generator", "feed", error);
+			self->priv->total_results = strtoul ((gchar*) total_results_string, NULL, 10);
+			xmlFree (total_results_string);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "startIndex") == 0) {
+			/* openSearch:startIndex */
+			xmlChar *start_index_string;
 
-		/* Parse the element's parameters */
-		name = xmlNodeListGetString (doc, node->children, TRUE);
-		uri = xmlGetProp (node, (xmlChar*) "uri");
-		version = xmlGetProp (node, (xmlChar*) "version");
+			/* Duplicate checking */
+			if (self->priv->start_index != 0)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		self->priv->generator = gdata_generator_new ((gchar*) name, (gchar*) uri, (gchar*) version);
+			/* Parse the number */
+			start_index_string = xmlNodeListGetString (doc, node->children, TRUE);
+			if (start_index_string == NULL)
+				return gdata_parser_error_required_content_missing (node, error);
 
-		xmlFree (name);
-		xmlFree (uri);
-		xmlFree (version);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "totalResults") == 0) {
-		/* openSearch:totalResults */
-		xmlChar *total_results_string;
+			self->priv->start_index = strtoul ((gchar*) start_index_string, NULL, 10);
+			xmlFree (start_index_string);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "itemsPerPage") == 0) {
+			/* openSearch:itemsPerPage */
+			xmlChar *items_per_page_string;
 
-		/* Duplicate checking */
-		if (self->priv->total_results != 0)
-			return gdata_parser_error_duplicate_element ("totalResults", "feed", error);
+			/* Duplicate checking */
+			if (self->priv->items_per_page != 0)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		/* Parse the number */
-		total_results_string = xmlNodeListGetString (doc, node->children, TRUE);
-		if (total_results_string == NULL)
-			return gdata_parser_error_required_content_missing ("openSearch:totalResults", error);
+			/* Parse the number */
+			items_per_page_string = xmlNodeListGetString (doc, node->children, TRUE);
+			if (items_per_page_string == NULL)
+				return gdata_parser_error_required_content_missing (node, error);
 
-		self->priv->total_results = strtoul ((gchar*) total_results_string, NULL, 10);
-		xmlFree (total_results_string);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "startIndex") == 0) {
-		/* openSearch:startIndex */
-		xmlChar *start_index_string;
-
-		/* Duplicate checking */
-		if (self->priv->start_index != 0)
-			return gdata_parser_error_duplicate_element ("startIndex", "feed", error);
-
-		/* Parse the number */
-		start_index_string = xmlNodeListGetString (doc, node->children, TRUE);
-		if (start_index_string == NULL)
-			return gdata_parser_error_required_content_missing ("openSearch:startIndex", error);
-
-		self->priv->start_index = strtoul ((gchar*) start_index_string, NULL, 10);
-		xmlFree (start_index_string);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "itemsPerPage") == 0) {
-		/* openSearch:itemsPerPage */
-		xmlChar *items_per_page_string;
-
-		/* Duplicate checking */
-		if (self->priv->items_per_page != 0)
-			return gdata_parser_error_duplicate_element ("itemsPerPage", "feed", error);
-
-		/* Parse the number */
-		items_per_page_string = xmlNodeListGetString (doc, node->children, TRUE);
-		if (items_per_page_string == NULL)
-			return gdata_parser_error_required_content_missing ("openSearch:itemsPerPage", error);
-
-		self->priv->items_per_page = strtoul ((gchar*) items_per_page_string, NULL, 10);
-		xmlFree (items_per_page_string);
-	} else if (GDATA_PARSABLE_CLASS (gdata_feed_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
-		/* Error! */
-		return FALSE;
+			self->priv->items_per_page = strtoul ((gchar*) items_per_page_string, NULL, 10);
+			xmlFree (items_per_page_string);
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_feed_parent_class)->parse_xml (parsable, doc, node, user_data, error);
+		}
+	} else {
+		return GDATA_PARSABLE_CLASS (gdata_feed_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
 
 	return TRUE;
@@ -613,24 +582,19 @@ post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error)
 
 GDataFeed *
 _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType entry_type,
-			  GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
+                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	ParseData *data;
 	GDataFeed *feed;
 
-	g_return_val_if_fail (g_type_is_a (feed_type, GDATA_TYPE_FEED) == TRUE, FALSE);
+	g_return_val_if_fail (g_type_is_a (feed_type, GDATA_TYPE_FEED), NULL);
 	g_return_val_if_fail (xml != NULL, NULL);
-	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY) == TRUE, FALSE);
+	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	data = g_slice_new (ParseData);
-	data->entry_type = entry_type;
-	data->progress_callback = progress_callback;
-	data->progress_user_data = progress_user_data;
-	data->entry_i = 0;
-
-	feed = GDATA_FEED (_gdata_parsable_new_from_xml (feed_type, "feed", xml, length, data, error));
-
-	g_slice_free (ParseData, data);
+	data = _gdata_feed_parse_data_new (entry_type, progress_callback, progress_user_data);
+	feed = GDATA_FEED (_gdata_parsable_new_from_xml (feed_type, xml, length, data, error));
+	_gdata_feed_parse_data_free (data);
 
 	return feed;
 }
@@ -641,7 +605,7 @@ _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType
  *
  * Returns a list of the entries contained in this feed.
  *
- * Return value: a #GList of #GDataEntry<!-- -->s; free the list with g_list_free(), but do not unref the entries
+ * Return value: (element-type GData.Entry) (transfer none): a #GList of #GDataEntry<!-- -->s
  **/
 GList *
 gdata_feed_get_entries (GDataFeed *self)
@@ -663,7 +627,7 @@ entry_compare_cb (const GDataEntry *entry, const gchar *id)
  *
  * Returns the entry in the feed with the given @id, if found.
  *
- * Return value: the #GDataEntry, or %NULL
+ * Return value: (transfer none): the #GDataEntry, or %NULL
  *
  * Since: 0.2.0
  **/
@@ -687,7 +651,7 @@ gdata_feed_look_up_entry (GDataFeed *self, const gchar *id)
  *
  * Returns a list of the categories listed in this feed.
  *
- * Return value: a #GList of #GDataCategory<!-- -->s; free the list with g_list_free(), but do not free the categories
+ * Return value: (element-type GData.Category) (transfer none): a #GList of #GDataCategory<!-- -->s
  **/
 GList *
 gdata_feed_get_categories (GDataFeed *self)
@@ -702,7 +666,7 @@ gdata_feed_get_categories (GDataFeed *self)
  *
  * Returns a list of the links listed in this feed.
  *
- * Return value: a #GList of #GDataLink<!-- -->s; free the list with g_list_free(), but do not free the links
+ * Return value: (element-type GData.Link) (transfer none): a #GList of #GDataLink<!-- -->s
  **/
 GList *
 gdata_feed_get_links (GDataFeed *self)
@@ -714,17 +678,17 @@ gdata_feed_get_links (GDataFeed *self)
 static gint
 link_compare_cb (const GDataLink *link, const gchar *rel)
 {
-	return strcmp (link->rel, rel);
+	return strcmp (gdata_link_get_relation_type ((GDataLink*) link), rel);
 }
 
 /**
  * gdata_feed_look_up_link:
  * @self: a #GDataFeed
- * @rel: the value of the <structfield>rel</structfield> attribute of the desired link
+ * @rel: the value of the #GDataLink:relation-type property of the desired link
  *
- * Looks up a link by <structfield>rel</structfield> value from the list of links in the feed.
+ * Looks up a link by #GDataLink:relation-type value from the list of links in the feed.
  *
- * Return value: a #GDataLink, or %NULL if one was not found
+ * Return value: (transfer none): a #GDataLink, or %NULL if one was not found
  *
  * Since: 0.1.1
  **/
@@ -739,7 +703,7 @@ gdata_feed_look_up_link (GDataFeed *self, const gchar *rel)
 	element = g_list_find_custom (self->priv->links, rel, (GCompareFunc) link_compare_cb);
 	if (element == NULL)
 		return NULL;
-	return (GDataLink*) (element->data);
+	return GDATA_LINK (element->data);
 }
 
 /**
@@ -748,7 +712,7 @@ gdata_feed_look_up_link (GDataFeed *self, const gchar *rel)
  *
  * Returns a list of the authors listed in this feed.
  *
- * Return value: a #GList of #GDataAuthor<!-- -->s; free the list with g_list_free(), but do not free the authors
+ * Return value: (element-type GData.Author) (transfer none): a #GList of #GDataAuthor<!-- -->s
  **/
 GList *
 gdata_feed_get_authors (GDataFeed *self)
@@ -822,7 +786,7 @@ gdata_feed_get_etag (GDataFeed *self)
 /**
  * gdata_feed_get_updated:
  * @self: a #GDataFeed
- * @updated: a #GTimeVal
+ * @updated: (out caller-allocates): a #GTimeVal
  *
  * Puts the time the feed was last updated into @updated.
  **/
@@ -849,13 +813,30 @@ gdata_feed_get_logo (GDataFeed *self)
 	return self->priv->logo;
 }
 
+/**
+ * gdata_feed_get_icon:
+ * @self: a #GDataFeed
+ *
+ * Returns the icon URI of the feed.
+ *
+ * Return value: the feed's icon URI, or %NULL
+ *
+ * Since: 0.6.0
+ **/
+const gchar *
+gdata_feed_get_icon (GDataFeed *self)
+{
+	g_return_val_if_fail (GDATA_IS_FEED (self), NULL);
+	return self->priv->icon;
+}
+
 /**
  * gdata_feed_get_generator:
  * @self: a #GDataFeed
  *
  * Returns details about the software which generated the feed.
  *
- * Return value: a #GDataGenerator, or %NULL
+ * Return value: (transfer none): a #GDataGenerator, or %NULL
  **/
 GDataGenerator *
 gdata_feed_get_generator (GDataFeed *self)
@@ -870,7 +851,7 @@ gdata_feed_get_generator (GDataFeed *self)
  *
  * Returns the number of items per results page feed.
  *
- * Return value: the number of items per results page feed, or %0
+ * Return value: the number of items per results page feed, or <code class="literal">0</code>
  **/
 guint
 gdata_feed_get_items_per_page (GDataFeed *self)
@@ -885,7 +866,7 @@ gdata_feed_get_items_per_page (GDataFeed *self)
  *
  * Returns the one-based start index of the results feed in the result set.
  *
- * Return value: the one-based start index, or %0
+ * Return value: the one-based start index, or <code class="literal">0</code>
  **/
 guint
 gdata_feed_get_start_index (GDataFeed *self)
@@ -901,7 +882,7 @@ gdata_feed_get_start_index (GDataFeed *self)
  * Returns the total number of results in the result set, including results on other
  * pages.
  *
- * Return value: the total number of results, or %0
+ * Return value: the total number of results, or <code class="literal">0</code>
  **/
 guint
 gdata_feed_get_total_results (GDataFeed *self)
@@ -909,3 +890,62 @@ gdata_feed_get_total_results (GDataFeed *self)
 	g_return_val_if_fail (GDATA_IS_FEED (self), 0);
 	return self->priv->total_results;
 }
+
+void
+_gdata_feed_add_entry (GDataFeed *self, GDataEntry *entry)
+{
+	g_return_if_fail (GDATA_IS_FEED (self));
+	g_return_if_fail (GDATA_IS_ENTRY (entry));
+	self->priv->entries = g_list_prepend (self->priv->entries, entry);
+}
+
+gpointer
+_gdata_feed_parse_data_new (GType entry_type, GDataQueryProgressCallback progress_callback, gpointer progress_user_data)
+{
+	ParseData *data;
+	data = g_slice_new (ParseData);
+	data->entry_type = entry_type;
+	data->progress_callback = progress_callback;
+	data->progress_user_data = progress_user_data;
+	data->entry_i = 0;
+	return data;
+}
+
+void
+_gdata_feed_parse_data_free (gpointer data)
+{
+	g_slice_free (ParseData, data);
+}
+
+static gboolean
+progress_callback_idle (ProgressCallbackData *data)
+{
+	data->progress_callback (data->entry, data->entry_i, data->total_results, data->progress_user_data);
+	g_object_unref (data->entry);
+	g_slice_free (ProgressCallbackData, data);
+	return FALSE;
+}
+
+void
+_gdata_feed_call_progress_callback (GDataFeed *self, gpointer user_data, GDataEntry *entry)
+{
+	ParseData *data = user_data;
+
+	if (data->progress_callback != NULL) {
+		ProgressCallbackData *progress_data;
+
+		/* Build the data for the callback */
+		progress_data = g_slice_new (ProgressCallbackData);
+		progress_data->progress_callback = data->progress_callback;
+		progress_data->progress_user_data = data->progress_user_data;
+		progress_data->entry = g_object_ref (entry);
+		progress_data->entry_i = data->entry_i;
+		progress_data->total_results = MIN (self->priv->items_per_page, self->priv->total_results);
+
+		/* Send the callback; use G_PRIORITY_DEFAULT rather than G_PRIORITY_DEFAULT_IDLE
+		* to contend with the priorities used by the callback functions in GAsyncResult */
+		g_idle_add_full (G_PRIORITY_DEFAULT, (GSourceFunc) progress_callback_idle, progress_data, NULL);
+	}
+	data->entry_i++;
+}
+
