@@ -334,7 +334,7 @@ gdata_upload_stream_write (GOutputStream *stream, const void *buffer, gsize coun
 
 	/* Listen for cancellation events */
 	if (cancellable != NULL)
-		cancelled_signal = g_signal_connect (cancellable, "cancelled", (GCallback) write_cancelled_cb, GDATA_UPLOAD_STREAM (stream));
+		cancelled_signal = g_cancellable_connect (cancellable, (GCallback) write_cancelled_cb, GDATA_UPLOAD_STREAM (stream), NULL);
 
 	/* Set write_finished so we know if the write operation has finished before we reach write_cond */
 	priv->write_finished = FALSE;
@@ -371,7 +371,7 @@ gdata_upload_stream_write (GOutputStream *stream, const void *buffer, gsize coun
 	create_network_thread (GDATA_UPLOAD_STREAM (stream), error);
 	if (priv->network_thread == NULL) {
 		if (cancellable != NULL)
-			g_signal_handler_disconnect (cancellable, cancelled_signal);
+			g_cancellable_disconnect (cancellable, cancelled_signal);
 		return -1;
 	}
 
@@ -386,7 +386,7 @@ write:
 
 	/* Disconnect from the cancelled signal so we can't receive any more cancel events before we handle errors */
 	if (cancellable != NULL)
-		g_signal_handler_disconnect (cancellable, cancelled_signal);
+		g_cancellable_disconnect (cancellable, cancelled_signal);
 
 	/* Check for an error and return if necessary */
 	if (priv->response_error != NULL) {
@@ -430,14 +430,14 @@ gdata_upload_stream_close (GOutputStream *stream, GCancellable *cancellable, GEr
 
 		/* Allow cancellation */
 		if (cancellable != NULL)
-			cancelled_signal = g_signal_connect (cancellable, "cancelled", (GCallback) close_cancelled_cb, GDATA_UPLOAD_STREAM (stream));
+			cancelled_signal = g_cancellable_connect (cancellable, (GCallback) close_cancelled_cb, GDATA_UPLOAD_STREAM (stream), NULL);
 
 		/* Wait for the signal that we've finished */
 		g_cond_wait (priv->finished_cond, g_static_mutex_get_mutex (&(priv->response_mutex)));
 
 		/* Disconnect from the signal handler so we can't receive any more cancellation events before we handle errors*/
 		if (cancellable != NULL)
-			g_signal_handler_disconnect (cancellable, cancelled_signal);
+			g_cancellable_disconnect (cancellable, cancelled_signal);
 	}
 
 	/* Report any errors which have been set by the network thread */
@@ -547,7 +547,7 @@ create_network_thread (GDataUploadStream *self, GError **error)
  * @service: a #GDataService
  * @method: the HTTP method to use
  * @upload_uri: the URI to upload
- * @entry: the entry to upload as metadata, or %NULL
+ * @entry: (allow-none): the entry to upload as metadata, or %NULL
  * @slug: the file's slug (filename)
  * @content_type: the content type of the file being uploaded
  *
@@ -626,7 +626,7 @@ gdata_upload_stream_new (GDataService *service, const gchar *method, const gchar
 /**
  * gdata_upload_stream_get_response:
  * @self: a #GDataUploadStream
- * @length: return location for the length of the response, or %NULL
+ * @length: (allow-none) (out caller-allocates): return location for the length of the response, or %NULL
  *
  * Returns the server's response to the upload operation performed by the #GDataUploadStream. If the operation
  * is still underway, or the server's response hasn't been received yet, %NULL is returned and @length is set to <code class="literal">-1</code>.
@@ -678,7 +678,7 @@ gdata_upload_stream_get_response (GDataUploadStream *self, gssize *length)
  *
  * Gets the service used to authenticate the upload, as passed to gdata_upload_stream_new().
  *
- * Return value: the #GDataService used to authenticate the upload
+ * Return value: (transfer none): the #GDataService used to authenticate the upload
  *
  * Since: 0.5.0
  **/
@@ -712,7 +712,7 @@ gdata_upload_stream_get_upload_uri (GDataUploadStream *self)
  *
  * Gets the entry being used to upload metadata, if one was passed to gdata_upload_stream_new().
  *
- * Return value: the entry used for metadata, or %NULL
+ * Return value: (transfer none): the entry used for metadata, or %NULL
  *
  * Since: 0.5.0
  **/
