@@ -447,15 +447,10 @@ gdata_calendar_event_new (const gchar *id)
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GDataCalendarEvent *self;
+	GDataCalendarEvent *self = GDATA_CALENDAR_EVENT (parsable);
 
-	g_return_val_if_fail (GDATA_IS_CALENDAR_EVENT (parsable), FALSE);
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (node != NULL, FALSE);
-
-	self = GDATA_CALENDAR_EVENT (parsable);
-
-	if (xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
+	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
+	    xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
 		/* app:edited */
 		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
@@ -465,128 +460,135 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			return FALSE;
 		}
 		xmlFree (edited);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "comments") == 0) {
-		/* gd:comments */
-		xmlChar *rel, *href, *count_hint;
-		xmlNode *child_node;
-		guint count_hint_uint;
-		/*GDataGDFeedLink *feed_link;*/
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "comments") == 0) {
+			/* gd:comments */
+			xmlChar *rel, *href, *count_hint;
+			xmlNode *child_node;
+			guint count_hint_uint;
+			/*GDataGDFeedLink *feed_link;*/
 
-		/* This is actually the child of the <comments> element */
-		child_node = node->children;
+			/* This is actually the child of the <comments> element */
+			child_node = node->children;
 
-		count_hint = xmlGetProp (child_node, (xmlChar*) "countHint");
-		if (count_hint == NULL)
-			count_hint_uint = 0;
-		else
-			count_hint_uint = strtoul ((gchar*) count_hint, NULL, 10);
-		xmlFree (count_hint);
+			count_hint = xmlGetProp (child_node, (xmlChar*) "countHint");
+			if (count_hint == NULL)
+				count_hint_uint = 0;
+			else
+				count_hint_uint = strtoul ((gchar*) count_hint, NULL, 10);
+			xmlFree (count_hint);
 
-		rel = xmlGetProp (child_node, (xmlChar*) "rel");
-		href = xmlGetProp (child_node, (xmlChar*) "href");
+			rel = xmlGetProp (child_node, (xmlChar*) "rel");
+			href = xmlGetProp (child_node, (xmlChar*) "href");
 
-		/* TODO */
-		/*feed_link = gdata_gd_feed_link_new ((gchar*) href, (gchar*) rel, count_hint_uint);*/
-		/*gdata_calendar_event_set_comments_feed_link (self, feed_link);*/
+			/* TODO */
+			/*feed_link = gdata_gd_feed_link_new ((gchar*) href, (gchar*) rel, count_hint_uint);*/
+			/*gdata_calendar_event_set_comments_feed_link (self, feed_link);*/
 
-		xmlFree (rel);
-		xmlFree (href);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "eventStatus") == 0) {
-		/* gd:eventStatus */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		self->priv->status = (gchar*) value;
-	} else if (xmlStrcmp (node->name, (xmlChar*) "visibility") == 0) {
-		/* gd:visibility */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		self->priv->visibility = (gchar*) value;
-	} else if (xmlStrcmp (node->name, (xmlChar*) "transparency") == 0) {
-		/* gd:transparency */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		self->priv->transparency = (gchar*) value;
-	} else if (xmlStrcmp (node->name, (xmlChar*) "uid") == 0) {
-		/* gCal:uid */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		self->priv->uid = (gchar*) value;
-	} else if (xmlStrcmp (node->name, (xmlChar*) "sequence") == 0) {
-		/* gCal:sequence */
-		xmlChar *value;
-		guint value_uint;
+			xmlFree (rel);
+			xmlFree (href);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "eventStatus") == 0) {
+			/* gd:eventStatus */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			self->priv->status = (gchar*) value;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "visibility") == 0) {
+			/* gd:visibility */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			self->priv->visibility = (gchar*) value;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "transparency") == 0) {
+			/* gd:transparency */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			self->priv->transparency = (gchar*) value;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "when") == 0) {
+			/* gd:when */
+			GDataGDWhen *when = GDATA_GD_WHEN (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHEN, doc, node, NULL, error));
+			if (when == NULL)
+				return FALSE;
 
-		value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		else
-			value_uint = strtoul ((gchar*) value, NULL, 10);
-		xmlFree (value);
+			gdata_calendar_event_add_time (self, when);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "who") == 0) {
+			/* gd:who */
+			GDataGDWho *who = GDATA_GD_WHO (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHO, doc, node, NULL, error));
+			if (who == NULL)
+				return FALSE;
 
-		gdata_calendar_event_set_sequence (self, value_uint);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "when") == 0) {
-		/* gd:when */
-		GDataGDWhen *when = GDATA_GD_WHEN (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHEN, doc, node, NULL, error));
-		if (when == NULL)
-			return FALSE;
+			gdata_calendar_event_add_person (self, who);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
+			/* gd:where */
+			GDataGDWhere *where = GDATA_GD_WHERE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHERE, doc, node, NULL, error));
+			if (where == NULL)
+				return FALSE;
 
-		gdata_calendar_event_add_time (self, when);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanModify") == 0) {
-		/* gCal:guestsCanModify */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		gdata_calendar_event_set_guests_can_modify (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-		xmlFree (value);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanInviteOthers") == 0) {
-		/* gCal:guestsCanInviteOthers */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		gdata_calendar_event_set_guests_can_invite_others (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-		xmlFree (value);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanSeeGuests") == 0) {
-		/* gCal:guestsCanSeeGuests */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		gdata_calendar_event_set_guests_can_see_guests (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-		xmlFree (value);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "anyoneCanAddSelf") == 0) {
-		/* gCal:anyoneCanAddSelf */
-		xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
-		if (value == NULL)
-			return gdata_parser_error_required_property_missing (node, "value", error);
-		gdata_calendar_event_set_anyone_can_add_self (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
-		xmlFree (value);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "who") == 0) {
-		/* gd:who */
-		GDataGDWho *who = GDATA_GD_WHO (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHO, doc, node, NULL, error));
-		if (who == NULL)
-			return FALSE;
+			gdata_calendar_event_add_place (self, where);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "recurrence") == 0) {
+			/* gd:recurrence */
+			self->priv->recurrence = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "originalEvent") == 0) {
+			/* gd:originalEvent */
+			self->priv->original_event_id = (gchar*) xmlGetProp (node, (xmlChar*) "id");
+			self->priv->original_event_uri = (gchar*) xmlGetProp (node, (xmlChar*) "href");
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->parse_xml (parsable, doc, node, user_data, error);
+		}
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/gCal/2005") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "uid") == 0) {
+			/* gCal:uid */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			self->priv->uid = (gchar*) value;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "sequence") == 0) {
+			/* gCal:sequence */
+			xmlChar *value;
+			guint value_uint;
 
-		gdata_calendar_event_add_person (self, who);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
-		/* gd:where */
-		GDataGDWhere *where = GDATA_GD_WHERE (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GD_WHERE, doc, node, NULL, error));
-		if (where == NULL)
-			return FALSE;
+			value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			else
+				value_uint = strtoul ((gchar*) value, NULL, 10);
+			xmlFree (value);
 
-		gdata_calendar_event_add_place (self, where);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "recurrence") == 0) {
-		/* gd:recurrence */
-		self->priv->recurrence = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "originalEvent") == 0) {
-		/* gd:originalEvent */
-		self->priv->original_event_id = (gchar*) xmlGetProp (node, (xmlChar*) "id");
-		self->priv->original_event_uri = (gchar*) xmlGetProp (node, (xmlChar*) "href");
-	} else if (GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
-		/* Error! */
-		return FALSE;
+			gdata_calendar_event_set_sequence (self, value_uint);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanModify") == 0) {
+			/* gCal:guestsCanModify */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			gdata_calendar_event_set_guests_can_modify (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
+			xmlFree (value);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanInviteOthers") == 0) {
+			/* gCal:guestsCanInviteOthers */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			gdata_calendar_event_set_guests_can_invite_others (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
+			xmlFree (value);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "guestsCanSeeGuests") == 0) {
+			/* gCal:guestsCanSeeGuests */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			gdata_calendar_event_set_guests_can_see_guests (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
+			xmlFree (value);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "anyoneCanAddSelf") == 0) {
+			/* gCal:anyoneCanAddSelf */
+			xmlChar *value = xmlGetProp (node, (xmlChar*) "value");
+			if (value == NULL)
+				return gdata_parser_error_required_property_missing (node, "value", error);
+			gdata_calendar_event_set_anyone_can_add_self (self, (xmlStrcmp (value, (xmlChar*) "true") == 0) ? TRUE : FALSE);
+			xmlFree (value);
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->parse_xml (parsable, doc, node, user_data, error);
+		}
+	} else {
+		return GDATA_PARSABLE_CLASS (gdata_calendar_event_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
 
 	return TRUE;
@@ -676,10 +678,10 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 /**
  * gdata_calendar_event_get_edited:
  * @self: a #GDataCalendarEvent
- * @edited: a #GTimeVal
+ * @edited: (out caller-allocates): a #GTimeVal
  *
  * Gets the #GDataCalendarEvent:edited property and puts it in @edited. If the property is unset,
- * both fields in the #GTimeVal will be set to %0.
+ * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
  **/
 void
 gdata_calendar_event_get_edited (GDataCalendarEvent *self, GTimeVal *edited)
@@ -709,7 +711,7 @@ gdata_calendar_event_get_status (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_set_status:
  * @self: a #GDataCalendarEvent
- * @status: a new event status, or %NULL
+ * @status: (allow-none): a new event status, or %NULL
  *
  * Sets the #GDataCalendarEvent:status property to the new status, @status.
  *
@@ -745,7 +747,7 @@ gdata_calendar_event_get_visibility (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_set_visibility:
  * @self: a #GDataCalendarEvent
- * @visibility: a new event visibility, or %NULL
+ * @visibility: (allow-none): a new event visibility, or %NULL
  *
  * Sets the #GDataCalendarEvent:visibility property to the new visibility, @visibility.
  *
@@ -779,7 +781,7 @@ gdata_calendar_event_get_transparency (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_set_transparency:
  * @self: a #GDataCalendarEvent
- * @transparency: a new event transparency, or %NULL
+ * @transparency: (allow-none): a new event transparency, or %NULL
  *
  * Sets the #GDataCalendarEvent:transparency property to the new transparency, @transparency.
  *
@@ -813,7 +815,7 @@ gdata_calendar_event_get_uid (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_set_uid:
  * @self: a #GDataCalendarEvent
- * @uid: a new event UID, or %NULL
+ * @uid: (allow-none): a new event UID, or %NULL
  *
  * Sets the #GDataCalendarEvent:uid property to the new UID, @uid.
  *
@@ -1005,7 +1007,7 @@ gdata_calendar_event_add_person (GDataCalendarEvent *self, GDataGDWho *who)
  *
  * Gets a list of the people attending the event.
  *
- * Return value: a #GList of #GDataGDWho<!-- -->s, or %NULL
+ * Return value: (element-type GData.GDWho) (transfer none): a #GList of #GDataGDWho<!-- -->s, or %NULL
  *
  * Since: 0.2.0
  **/
@@ -1041,7 +1043,7 @@ gdata_calendar_event_add_place (GDataCalendarEvent *self, GDataGDWhere *where)
  *
  * Gets a list of the locations associated with the event.
  *
- * Return value: a #GList of #GDataGDWhere<!-- -->s, or %NULL
+ * Return value: (element-type GData.GDWhere) (transfer none): a #GList of #GDataGDWhere<!-- -->s, or %NULL
  *
  * Since: 0.2.0
  **/
@@ -1079,7 +1081,7 @@ gdata_calendar_event_add_time (GDataCalendarEvent *self, GDataGDWhen *when)
  *
  * Gets a list of the time periods associated with the event.
  *
- * Return value: a #GList of #GDataGDWhen<!-- -->s, or %NULL
+ * Return value: (element-type GData.GDWhen) (transfer none): a #GList of #GDataGDWhen<!-- -->s, or %NULL
  *
  * Since: 0.2.0
  **/
@@ -1093,9 +1095,9 @@ gdata_calendar_event_get_times (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_get_primary_time:
  * @self: a #GDataCalendarEvent
- * @start_time: a #GTimeVal for the start time, or %NULL
- * @end_time: a #GTimeVal for the end time, or %NULL
- * @when: a #GDataGDWhen for the primary time structure, or %NULL
+ * @start_time: (out caller-allocates): a #GTimeVal for the start time, or %NULL
+ * @end_time: (out caller-allocates): a #GTimeVal for the end time, or %NULL
+ * @when: (out callee-allocates) (transfer none): a #GDataGDWhen for the primary time structure, or %NULL
  *
  * Gets the first time period associated with the event, conveniently returning just its start and
  * end times if required.
@@ -1148,7 +1150,7 @@ gdata_calendar_event_get_recurrence (GDataCalendarEvent *self)
 /**
  * gdata_calendar_event_set_recurrence:
  * @self: a #GDataCalendarEvent
- * @recurrence: a new event recurrence, or %NULL
+ * @recurrence: (allow-none): a new event recurrence, or %NULL
  *
  * Sets the #GDataCalendarEvent:recurrence property to the new recurrence, @recurrence.
  *
@@ -1169,8 +1171,8 @@ gdata_calendar_event_set_recurrence (GDataCalendarEvent *self, const gchar *recu
 /**
  * gdata_calendar_event_get_original_event_details:
  * @self: a #GDataCalendarEvent
- * @event_id: return location for the original event's ID, or %NULL
- * @event_uri: return location for the original event's URI, or %NULL
+ * @event_id: (out callee-allocates) (transfer full): return location for the original event's ID, or %NULL
+ * @event_uri: (out callee-allocates) (transfer full): return location for the original event's URI, or %NULL
  *
  * Gets details of the original event, if this event is an exception to a recurring event. The original
  * event's ID and the URI of the event's XML are returned in @event_id and @event_uri, respectively.
