@@ -2,6 +2,7 @@
 /*
  * GData Client
  * Copyright (C) Thibault Saunier 2009 <saunierthibault@gmail.com>
+ * Copyright (C) Philip Withnall 2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,8 @@
  * Fore more details about the spreadsheet downloads handling, see the
  * <ulink type="http" url="http://groups.google.com/group/Google-Docs-Data-APIs/browse_thread/thread/bfc50e94e303a29a?pli=1">
  * online explanation about the problem</ulink>.
+ *
+ * Since: 0.4.0
  **/
 
 #include <config.h>
@@ -163,17 +166,17 @@ gdata_documents_service_get_property (GObject *object, guint property_id, GValue
 /**
  * gdata_documents_service_query_documents:
  * @self: a #GDataDocumentsService
- * @query: a #GDataQuery with the query parameters, or %NULL
+ * @query: (allow-none): a #GDataDocumentsQuery with the query parameters, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
- * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
- * @progress_user_data: data to pass to the @progress_callback function
+ * @progress_callback: (scope call): a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
+ * @progress_user_data: (closure): data to pass to the @progress_callback function
  * @error: a #GError, or %NULL
  *
  * Queries the service to return a list of documents matching the given @query.
  *
  * For more details, see gdata_service_query().
  *
- * Return value: a #GDataDocumentsFeed of query results; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataDocumentsFeed of query results; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
@@ -218,11 +221,11 @@ gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocum
  * @document_type should be the expected type of the document to be returned. e.g. %GDATA_TYPE_DOCUMENTS_SPREADSHEET if you're querying
  * for a spreadsheet.
  *
- * @document_id should be the ID of the document as returned by gdata_document_entry_get_document_id().
+ * @document_id should be the ID of the document as returned by gdata_documents_entry_get_document_id().
  *
  * Parameters and errors are as for gdata_service_query().
  *
- * Return value: a #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): a #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.5.0
  **/
@@ -272,14 +275,14 @@ gdata_documents_service_query_single_document (GDataDocumentsService *self, GTyp
 }
 
 /**
- * gdata_documents_service_query_documents_async:
+ * gdata_documents_service_query_documents_async: (skip)
  * @self: a #GDataDocumentsService
- * @query: a #GDataQuery with the query parameters, or %NULL
+ * @query: (allow-none): a #GDataDocumentsQuery with the query parameters, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: data to pass to the @progress_callback function
  * @callback: a #GAsyncReadyCallback to call when authentication is finished
- * @user_data: data to pass to the @callback function
+ * @user_data: (closure): data to pass to the @callback function
  *
  * Queries the service to return a list of documents matching the given @query. @self and
  * @query are both reffed when this function is called, so can safely be unreffed after this function returns.
@@ -441,9 +444,9 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
 /**
  * gdata_documents_service_upload_document:
  * @self: an authenticated #GDataDocumentsService
- * @document: the #GDataDocumentsEntry to insert, or %NULL
- * @document_file: the document to upload, or %NULL
- * @folder: the folder to which the document should be uploaded, or %NULL
+ * @document: (allow-none): the #GDataDocumentsEntry to insert, or %NULL
+ * @document_file: (allow-none): the document to upload, or %NULL
+ * @folder: (allow-none): the folder to which the document should be uploaded, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -460,7 +463,7 @@ upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *docume
  * If there is a problem reading @document_file, an error from g_file_load_contents() or g_file_query_info() will be returned. Other errors from
  * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
@@ -507,7 +510,7 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  * gdata_documents_service_update_document:
  * @self: a #GDataDocumentsService
  * @document: the #GDataDocumentsEntry to update
- * @document_file: the local document file containing the new data, or %NULL
+ * @document_file: (allow-none): the local document file containing the new data, or %NULL
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -519,7 +522,7 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  *
  * For more details, see gdata_service_insert_entry().
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
@@ -562,7 +565,7 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
  *
  * Errors from #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
@@ -650,8 +653,8 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 /**
  * gdata_documents_service_remove_document_from_folder:
  * @self: a #GDataDocumentsService
- * @document : the #GDataDocumentsEntry to remove
- * @folder : the #GDataDocumentsFolder from wich we should remove @document
+ * @document: the #GDataDocumentsEntry to remove
+ * @folder: the #GDataDocumentsFolder from which we should remove @document
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
@@ -659,7 +662,7 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
  *
  * Errors from #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
  *
- * Return value: an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ * Return value: (transfer full): an updated #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  **/
@@ -667,7 +670,7 @@ GDataDocumentsEntry *
 gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self, GDataDocumentsEntry *document, GDataDocumentsFolder *folder,
 						     GCancellable *cancellable, GError **error)
 {
-	const gchar *folder_id, *document_id, *uri_template;
+	const gchar *folder_id, *document_id;
 	GDataServiceClass *klass;
 	SoupMessage *message;
 	guint status;
@@ -690,18 +693,22 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 	g_assert (folder_id != NULL);
 	g_assert (document_id != NULL);
 
-	if (GDATA_IS_DOCUMENTS_PRESENTATION (document))
-		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s";
-	else if (GDATA_IS_DOCUMENTS_SPREADSHEET (document))
-		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s";
-	else if (GDATA_IS_DOCUMENTS_TEXT (document))
-		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s";
-	else if (GDATA_IS_DOCUMENTS_FOLDER (document))
-		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/folder%%3A%s";
-	else
+	if (GDATA_IS_DOCUMENTS_PRESENTATION (document)) {
+		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s",
+		                       _gdata_service_get_scheme (), folder_id, document_id);
+	} else if (GDATA_IS_DOCUMENTS_SPREADSHEET (document)) {
+		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s",
+		                       _gdata_service_get_scheme (), folder_id, document_id);
+	} else if (GDATA_IS_DOCUMENTS_TEXT (document)) {
+		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s",
+		                       _gdata_service_get_scheme (), folder_id, document_id);
+	} else if (GDATA_IS_DOCUMENTS_FOLDER (document)) {
+		uri = g_strdup_printf ("%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/folder%%3A%s",
+		                       _gdata_service_get_scheme (), folder_id, document_id);
+	} else {
 		g_assert_not_reached ();
+	}
 
-	uri = g_strdup_printf (uri_template, _gdata_service_get_scheme (), folder_id, document_id);
 	message = soup_message_new (SOUP_METHOD_DELETE, uri);
 	g_free (uri);
 
@@ -744,7 +751,7 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 
 /**
  * gdata_documents_service_get_upload_uri:
- * @folder: the #GDataDocumentsFolder into which to upload the document, or %NULL
+ * @folder: (allow-none): the #GDataDocumentsFolder into which to upload the document, or %NULL
  *
  * Gets the upload URI for documents for the service.
  *
