@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2009–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,12 +24,8 @@
 #include "gdata.h"
 #include "common.h"
 
-/* TODO: probably a better way to do this; some kind of data associated with the test suite? */
-static GDataService *service = NULL;
-static GMainLoop *main_loop = NULL;
-
 static GDataCalendarCalendar *
-get_calendar (GError **error)
+get_calendar (gconstpointer service, GError **error)
 {
 	GDataFeed *calendar_feed;
 	GDataCalendarCalendar *calendar;
@@ -56,6 +52,7 @@ static void
 test_authentication (void)
 {
 	gboolean retval;
+	GDataService *service;
 	GError *error = NULL;
 
 	/* Create a service */
@@ -75,10 +72,12 @@ test_authentication (void)
 	g_assert (gdata_service_is_authenticated (service) == TRUE);
 	g_assert_cmpstr (gdata_service_get_username (service), ==, USERNAME);
 	g_assert_cmpstr (gdata_service_get_password (service), ==, PASSWORD);
+
+	g_object_unref (service);
 }
 
 static void
-test_authentication_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_authentication_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	gboolean retval;
 	GError *error = NULL;
@@ -99,27 +98,29 @@ test_authentication_async_cb (GDataService *service, GAsyncResult *async_result,
 static void
 test_authentication_async (void)
 {
+	GDataService *service;
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
+
 	/* Create a service */
 	service = GDATA_SERVICE (gdata_calendar_service_new (CLIENT_ID));
 
 	g_assert (service != NULL);
 	g_assert (GDATA_IS_SERVICE (service));
 
-	gdata_service_authenticate_async (service, USERNAME, PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, NULL);
+	gdata_service_authenticate_async (service, USERNAME, PASSWORD, NULL, (GAsyncReadyCallback) test_authentication_async_cb, main_loop);
 
-	main_loop = g_main_loop_new (NULL, TRUE);
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
+
+	g_object_unref (service);
 }
 
 static void
-test_query_all_calendars (void)
+test_query_all_calendars (gconstpointer service)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	feed = gdata_calendar_service_query_all_calendars (GDATA_CALENDAR_SERVICE (service), NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_CALENDAR_FEED (feed));
@@ -131,7 +132,7 @@ test_query_all_calendars (void)
 }
 
 static void
-test_query_all_calendars_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_query_all_calendars_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
@@ -148,26 +149,23 @@ test_query_all_calendars_async_cb (GDataService *service, GAsyncResult *async_re
 }
 
 static void
-test_query_all_calendars_async (void)
+test_query_all_calendars_async (gconstpointer service)
 {
-	g_assert (service != NULL);
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
 	gdata_calendar_service_query_all_calendars_async (GDATA_CALENDAR_SERVICE (service), NULL, NULL, NULL,
-							  NULL, (GAsyncReadyCallback) test_query_all_calendars_async_cb, NULL);
+							  NULL, (GAsyncReadyCallback) test_query_all_calendars_async_cb, main_loop);
 
-	main_loop = g_main_loop_new (NULL, TRUE);
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 }
 
 static void
-test_query_own_calendars (void)
+test_query_own_calendars (gconstpointer service)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	feed = gdata_calendar_service_query_own_calendars (GDATA_CALENDAR_SERVICE (service), NULL, NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_CALENDAR_FEED (feed));
@@ -179,7 +177,7 @@ test_query_own_calendars (void)
 }
 
 static void
-test_query_own_calendars_async_cb (GDataService *service, GAsyncResult *async_result, gpointer user_data)
+test_query_own_calendars_async_cb (GDataService *service, GAsyncResult *async_result, GMainLoop *main_loop)
 {
 	GDataFeed *feed;
 	GError *error = NULL;
@@ -196,28 +194,25 @@ test_query_own_calendars_async_cb (GDataService *service, GAsyncResult *async_re
 }
 
 static void
-test_query_own_calendars_async (void)
+test_query_own_calendars_async (gconstpointer service)
 {
-	g_assert (service != NULL);
+	GMainLoop *main_loop = g_main_loop_new (NULL, TRUE);
 
 	gdata_calendar_service_query_own_calendars_async (GDATA_CALENDAR_SERVICE (service), NULL, NULL, NULL,
-							  NULL, (GAsyncReadyCallback) test_query_own_calendars_async_cb, NULL);
+							  NULL, (GAsyncReadyCallback) test_query_own_calendars_async_cb, main_loop);
 
-	main_loop = g_main_loop_new (NULL, TRUE);
 	g_main_loop_run (main_loop);
 	g_main_loop_unref (main_loop);
 }
 
 static void
-test_query_events (void)
+test_query_events (gconstpointer service)
 {
 	GDataFeed *feed;
 	GDataCalendarCalendar *calendar;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
-	calendar = get_calendar (&error);
+	calendar = get_calendar (service, &error);
 
 	/* Get the entry feed */
 	feed = gdata_calendar_service_query_events (GDATA_CALENDAR_SERVICE (service), calendar, NULL, NULL, NULL, NULL, &error);
@@ -232,7 +227,7 @@ test_query_events (void)
 }
 
 static void
-test_insert_simple (void)
+test_insert_simple (gconstpointer service)
 {
 	GDataCalendarEvent *event, *new_event;
 	GDataCategory *category;
@@ -240,31 +235,31 @@ test_insert_simple (void)
 	GDataGDWho *who;
 	GDataGDWhen *when;
 	GTimeVal start_time, end_time;
-	gchar *xml;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
 	event = gdata_calendar_event_new (NULL);
 
 	gdata_entry_set_title (GDATA_ENTRY (event), "Tennis with Beth");
 	gdata_entry_set_content (GDATA_ENTRY (event), "Meet for a quick lesson.");
 	category = gdata_category_new ("http://schemas.google.com/g/2005#event", "http://schemas.google.com/g/2005#kind", NULL);
 	gdata_entry_add_category (GDATA_ENTRY (event), category);
+	g_object_unref (category);
 	gdata_calendar_event_set_transparency (event, "http://schemas.google.com/g/2005#event.opaque");
 	gdata_calendar_event_set_status (event, "http://schemas.google.com/g/2005#event.confirmed");
 	where = gdata_gd_where_new (NULL, "Rolling Lawn Courts", NULL);
 	gdata_calendar_event_add_place (event, where);
+	g_object_unref (where);
 	who = gdata_gd_who_new ("http://schemas.google.com/g/2005#event.organizer", "John Smith‽", "john.smith@example.com");
 	gdata_calendar_event_add_person (event, who);
+	g_object_unref (who);
 	g_time_val_from_iso8601 ("2009-04-17T15:00:00.000Z", &start_time);
 	g_time_val_from_iso8601 ("2009-04-17T17:00:00.000Z", &end_time);
-	when = gdata_gd_when_new (&start_time, &end_time, FALSE, NULL, NULL);
+	when = gdata_gd_when_new (&start_time, &end_time, FALSE);
 	gdata_calendar_event_add_time (event, when);
+	g_object_unref (when);
 
 	/* Check the XML */
-	xml = gdata_entry_get_xml (GDATA_ENTRY (event));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (event,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -284,7 +279,6 @@ test_insert_simple (void)
 			 		"valueString='John Smith\342\200\275'/>"
 			 	"<gd:where valueString='Rolling Lawn Courts'/>"
 			 "</entry>");
-	g_free (xml);
 
 	/* Insert the event */
 	new_event = gdata_calendar_service_insert_event (GDATA_CALENDAR_SERVICE (service), event, NULL, &error);
@@ -299,15 +293,15 @@ test_insert_simple (void)
 }
 
 static void
-test_xml_dates (void)
+test_xml_dates (gconstpointer service)
 {
 	GDataCalendarEvent *event;
 	GList *times, *i;
 	GDataGDWhen *when;
-	gchar *xml;
+	GTimeVal time_val;
 	GError *error = NULL;
 
-	event = gdata_calendar_event_new_from_xml (
+	event = GDATA_CALENDAR_EVENT (gdata_parsable_new_from_xml (GDATA_TYPE_CALENDAR_EVENT,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 		 	"xmlns:gd='http://schemas.google.com/g/2005' "
 		 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -318,7 +312,7 @@ test_xml_dates (void)
 		 	"<gd:when startTime='2009-04-17'/>"
 		 	"<gd:when startTime='2009-04-17T15:00:00Z'/>"
 		 	"<gd:when startTime='2009-04-27' endTime='20090506'/>"
-		 "</entry>", -1, &error);
+		 "</entry>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ENTRY (event));
 	g_clear_error (&error);
@@ -327,43 +321,48 @@ test_xml_dates (void)
 	times = i = gdata_calendar_event_get_times (event);
 
 	/* First time */
-	when = (GDataGDWhen*) i->data;
+	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next != NULL);
-	g_assert (when->is_date == TRUE);
-	g_assert_cmpint (when->start_time.tv_sec, ==, 1239926400);
-	g_assert_cmpint (when->start_time.tv_usec, ==, 0);
-	g_assert_cmpint (when->end_time.tv_sec, ==, 0);
-	g_assert_cmpint (when->end_time.tv_usec, ==, 0);
-	g_assert (when->value_string == NULL);
-	g_assert (when->reminders == NULL);
+	g_assert (gdata_gd_when_is_date (when) == TRUE);
+	gdata_gd_when_get_start_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 1239926400);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	gdata_gd_when_get_end_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 0);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	g_assert (gdata_gd_when_get_value_string (when) == NULL);
+	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
 	/* Second time */
 	i = i->next;
-	when = (GDataGDWhen*) i->data;
+	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next != NULL);
-	g_assert (when->is_date == FALSE);
-	g_assert_cmpint (when->start_time.tv_sec, ==, 1239926400 + 54000);
-	g_assert_cmpint (when->start_time.tv_usec, ==, 0);
-	g_assert_cmpint (when->end_time.tv_sec, ==, 0);
-	g_assert_cmpint (when->end_time.tv_usec, ==, 0);
-	g_assert (when->value_string == NULL);
-	g_assert (when->reminders == NULL);
+	g_assert (gdata_gd_when_is_date (when) == FALSE);
+	gdata_gd_when_get_start_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 1239926400 + 54000);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	gdata_gd_when_get_end_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 0);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	g_assert (gdata_gd_when_get_value_string (when) == NULL);
+	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
 	/* Third time */
 	i = i->next;
-	when = (GDataGDWhen*) i->data;
+	when = GDATA_GD_WHEN (i->data);
 	g_assert (i->next == NULL);
-	g_assert (when->is_date == TRUE);
-	g_assert_cmpint (when->start_time.tv_sec, ==, 1239926400 + 864000);
-	g_assert_cmpint (when->start_time.tv_usec, ==, 0);
-	g_assert_cmpint (when->end_time.tv_sec, ==, 1241568000);
-	g_assert_cmpint (when->end_time.tv_usec, ==, 0);
-	g_assert (when->value_string == NULL);
-	g_assert (when->reminders == NULL);
+	g_assert (gdata_gd_when_is_date (when) == TRUE);
+	gdata_gd_when_get_start_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 1239926400 + 864000);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	gdata_gd_when_get_end_time (when, &time_val);
+	g_assert_cmpint (time_val.tv_sec, ==, 1241568000);
+	g_assert_cmpint (time_val.tv_usec, ==, 0);
+	g_assert (gdata_gd_when_get_value_string (when) == NULL);
+	g_assert (gdata_gd_when_get_reminders (when) == NULL);
 
 	/* Check the XML */
-	xml = gdata_entry_get_xml (GDATA_ENTRY (event));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (event,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -379,19 +378,18 @@ test_xml_dates (void)
 			 	"<gd:when startTime='2009-04-17T15:00:00Z'/>"
 			 	"<gd:when startTime='2009-04-27' endTime='2009-05-06'/>"
 			 "</entry>");
-	g_free (xml);
 
 	g_object_unref (event);
 }
 
 static void
-test_xml_recurrence (void)
+test_xml_recurrence (gconstpointer service)
 {
 	GDataCalendarEvent *event;
 	GError *error = NULL;
 	gchar *id, *uri;
 
-	event = gdata_calendar_event_new_from_xml (
+	event = GDATA_CALENDAR_EVENT (gdata_parsable_new_from_xml (GDATA_TYPE_CALENDAR_EVENT,
 		"<entry xmlns='http://www.w3.org/2005/Atom' "
 		 	"xmlns:gd='http://schemas.google.com/g/2005' "
 		 	"xmlns:gCal='http://schemas.google.com/gCal/2005' "
@@ -403,9 +401,9 @@ test_xml_recurrence (void)
 			"<category scheme='http://schemas.google.com/g/2005#kind' term='http://schemas.google.com/g/2005#event'/>"
 			"<title>Test daily instance event</title>"
 			"<content></content>"
-			"<link rel='alternate' type='text/html' href='http://www.google.com/calendar/event?eid=ZzU5MjhlODJycmNoOTViMjVmOHVkMGRsc2dfMjAwOTA0MjlUMTUzMDAwWiBsaWJnZGF0YS50ZXN0QGdvb2dsZW1haWwuY29t' title='alternate'/>"
-			"<link rel='self' type='application/atom+xml' href='http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg_20090429T153000Z'/>"
-			"<link rel='edit' type='application/atom+xml' href='http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg_20090429T153000Z'/>"
+			"<link rel='http://www.iana.org/assignments/relation/alternate' type='text/html' href='http://www.google.com/calendar/event?eid=ZzU5MjhlODJycmNoOTViMjVmOHVkMGRsc2dfMjAwOTA0MjlUMTUzMDAwWiBsaWJnZGF0YS50ZXN0QGdvb2dsZW1haWwuY29t' title='alternate'/>"
+			"<link rel='http://www.iana.org/assignments/relation/self' type='application/atom+xml' href='http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg_20090429T153000Z'/>"
+			"<link rel='http://www.iana.org/assignments/relation/edit' type='application/atom+xml' href='http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg_20090429T153000Z'/>"
 			"<author>"
 				"<name>GData Test</name>"
 				"<email>libgdata.test@googlemail.com</email>"
@@ -431,7 +429,7 @@ test_xml_recurrence (void)
 			"</gd:when>"
 			"<gd:who rel='http://schemas.google.com/g/2005#event.organizer' valueString='GData Test' email='libgdata.test@googlemail.com'/>"
 			"<gd:where valueString=''/>"
-		"</entry>", -1, &error);
+		"</entry>", -1, &error));
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ENTRY (event));
 	g_clear_error (&error);
@@ -443,6 +441,8 @@ test_xml_recurrence (void)
 	g_assert_cmpstr (id, ==, "g5928e82rrch95b25f8ud0dlsg");
 	g_assert_cmpstr (uri, ==, "http://www.google.com/calendar/feeds/libgdata.test@googlemail.com/private/full/g5928e82rrch95b25f8ud0dlsg");
 
+	g_free (id);
+	g_free (uri);
 	g_object_unref (event);
 }
 
@@ -518,18 +518,44 @@ test_query_uri (void)
 }
 
 static void
-test_acls_get_rules (void)
+test_query_etag (void)
+{
+	GDataCalendarQuery *query = gdata_calendar_query_new (NULL);
+
+	/* Test that setting any property will unset the ETag */
+	g_test_bug ("613529");
+
+#define CHECK_ETAG(C) \
+	gdata_query_set_etag (GDATA_QUERY (query), "foobar");		\
+	(C);								\
+	g_assert (gdata_query_get_etag (GDATA_QUERY (query)) == NULL);
+
+	CHECK_ETAG (gdata_calendar_query_set_future_events (query, FALSE))
+	CHECK_ETAG (gdata_calendar_query_set_order_by (query, "shizzle"))
+	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_start (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_recurrence_expansion_end (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_single_events (query, FALSE))
+	CHECK_ETAG (gdata_calendar_query_set_sort_order (query, "shizzle"))
+	CHECK_ETAG (gdata_calendar_query_set_start_min (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_start_max (query, NULL))
+	CHECK_ETAG (gdata_calendar_query_set_timezone (query, "about now"))
+
+#undef CHECK_ETAG
+
+	g_object_unref (query);
+}
+
+static void
+test_acls_get_rules (gconstpointer service)
 {
 	GDataFeed *feed;
 	GDataCalendarCalendar *calendar;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
-	calendar = get_calendar (&error);
+	calendar = get_calendar (service, &error);
 
 	/* Get the rules */
-	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), service, NULL, NULL, NULL, &error);
+	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_FEED (feed));
 	g_clear_error (&error);
@@ -541,19 +567,16 @@ test_acls_get_rules (void)
 }
 
 static void
-test_acls_insert_rule (void)
+test_acls_insert_rule (gconstpointer service)
 {
 	GDataCalendarCalendar *calendar;
 	GDataAccessRule *rule, *new_rule;
 	const gchar *scope_type, *scope_value;
 	GDataCategory *category;
 	GList *categories;
-	gchar *xml;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
-	calendar = get_calendar (&error);
+	calendar = get_calendar (service, &error);
 
 	rule = gdata_access_rule_new (NULL);
 
@@ -566,8 +589,7 @@ test_acls_insert_rule (void)
 	g_assert_cmpstr (scope_value, ==, "darcy@gmail.com");
 
 	/* Check the XML */
-	xml = gdata_entry_get_xml (GDATA_ENTRY (rule));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (rule,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 			 	"xmlns:gd='http://schemas.google.com/g/2005' "
 			 	"xmlns:gAcl='http://schemas.google.com/acl/2007'>"
@@ -576,10 +598,9 @@ test_acls_insert_rule (void)
 				"<gAcl:role value='http://schemas.google.com/gCal/2005#editor'/>"
 				"<gAcl:scope type='user' value='darcy@gmail.com'/>"
 			 "</entry>");
-	g_free (xml);
 
 	/* Insert the rule */
-	new_rule = gdata_access_handler_insert_rule (GDATA_ACCESS_HANDLER (calendar), service, rule, NULL, &error);
+	new_rule = gdata_access_handler_insert_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ACCESS_RULE (new_rule));
 	g_clear_error (&error);
@@ -595,18 +616,19 @@ test_acls_insert_rule (void)
 	g_assert (categories != NULL);
 	g_assert_cmpuint (g_list_length (categories), ==, 1);
 	category = categories->data;
-	g_assert_cmpstr (category->term, ==, "http://schemas.google.com/acl/2007#accessRule");
-	g_assert_cmpstr (category->scheme, ==, "http://schemas.google.com/g/2005#kind");
-	g_assert (category->label == NULL);
+	g_assert_cmpstr (gdata_category_get_term (category), ==, "http://schemas.google.com/acl/2007#accessRule");
+	g_assert_cmpstr (gdata_category_get_scheme (category), ==, "http://schemas.google.com/g/2005#kind");
+	g_assert (gdata_category_get_label (category) == NULL);
 
 	/* TODO: Check more properties? */
 
 	g_object_unref (rule);
 	g_object_unref (new_rule);
+	g_object_unref (calendar);
 }
 
 static void
-test_acls_update_rule (void)
+test_acls_update_rule (gconstpointer service)
 {
 	GDataFeed *feed;
 	GDataCalendarCalendar *calendar;
@@ -615,12 +637,10 @@ test_acls_update_rule (void)
 	GList *rules;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
-	calendar = get_calendar (&error);
+	calendar = get_calendar (service, &error);
 
 	/* Get a rule */
-	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), service, NULL, NULL, NULL, &error);
+	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_FEED (feed));
 	g_clear_error (&error);
@@ -643,7 +663,7 @@ test_acls_update_rule (void)
 	g_assert_cmpstr (gdata_access_rule_get_role (rule), ==, "http://schemas.google.com/gCal/2005#read");
 
 	/* Send the update to the server */
-	new_rule = gdata_access_handler_update_rule (GDATA_ACCESS_HANDLER (calendar), service, rule, NULL, &error);
+	new_rule = gdata_access_handler_update_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_ACCESS_RULE (new_rule));
 	g_clear_error (&error);
@@ -660,7 +680,7 @@ test_acls_update_rule (void)
 }
 
 static void
-test_acls_delete_rule (void)
+test_acls_delete_rule (gconstpointer service)
 {
 	GDataFeed *feed;
 	GDataCalendarCalendar *calendar;
@@ -669,12 +689,10 @@ test_acls_delete_rule (void)
 	gboolean success;
 	GError *error = NULL;
 
-	g_assert (service != NULL);
-
-	calendar = get_calendar (&error);
+	calendar = get_calendar (service, &error);
 
 	/* Get a rule */
-	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), service, NULL, NULL, NULL, &error);
+	feed = gdata_access_handler_get_rules (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), NULL, NULL, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_FEED (feed));
 	g_clear_error (&error);
@@ -695,49 +713,54 @@ test_acls_delete_rule (void)
 	g_object_unref (feed);
 
 	/* Delete the rule */
-	success = gdata_access_handler_delete_rule (GDATA_ACCESS_HANDLER (calendar), service, rule, NULL, &error);
+	success = gdata_access_handler_delete_rule (GDATA_ACCESS_HANDLER (calendar), GDATA_SERVICE (service), rule, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (success == TRUE);
 	g_clear_error (&error);
 
 	g_object_unref (rule);
+	g_object_unref (calendar);
 }
 
 int
 main (int argc, char *argv[])
 {
 	gint retval;
+	GDataService *service;
 
 	g_type_init ();
 	g_thread_init (NULL);
 	g_test_init (&argc, &argv, NULL);
 	g_test_bug_base ("http://bugzilla.gnome.org/show_bug.cgi?id=");
 
+	service = GDATA_SERVICE (gdata_calendar_service_new (CLIENT_ID));
+	gdata_service_authenticate (service, USERNAME, PASSWORD, NULL, NULL);
+
 	g_test_add_func ("/calendar/authentication", test_authentication);
 	if (g_test_thorough () == TRUE)
 		g_test_add_func ("/calendar/authentication_async", test_authentication_async);
-	g_test_add_func ("/calendar/query/all_calendars", test_query_all_calendars);
+	g_test_add_data_func ("/calendar/query/all_calendars", service, test_query_all_calendars);
 	if (g_test_thorough () == TRUE)
-		g_test_add_func ("/calendar/query/all_calendars_async", test_query_all_calendars_async);
-	g_test_add_func ("/calendar/query/own_calendars", test_query_own_calendars);
+		g_test_add_data_func ("/calendar/query/all_calendars_async", service, test_query_all_calendars_async);
+	g_test_add_data_func ("/calendar/query/own_calendars", service, test_query_own_calendars);
 	if (g_test_thorough () == TRUE)
-		g_test_add_func ("/calendar/query/own_calendars_async", test_query_own_calendars_async);
-	g_test_add_func ("/calendar/query/events", test_query_events);
+		g_test_add_data_func ("/calendar/query/own_calendars_async", service, test_query_own_calendars_async);
+	g_test_add_data_func ("/calendar/query/events", service, test_query_events);
 	if (g_test_slow () == TRUE)
-		g_test_add_func ("/calendar/insert/simple", test_insert_simple);
-	g_test_add_func ("/calendar/xml/dates", test_xml_dates);
-	g_test_add_func ("/calendar/xml/recurrence", test_xml_recurrence);
+		g_test_add_data_func ("/calendar/insert/simple", service, test_insert_simple);
+	g_test_add_data_func ("/calendar/xml/dates", service, test_xml_dates);
+	g_test_add_data_func ("/calendar/xml/recurrence", service, test_xml_recurrence);
 	g_test_add_func ("/calendar/query/uri", test_query_uri);
-	g_test_add_func ("/calendar/acls/get_rules", test_acls_get_rules);
+	g_test_add_func ("/calendar/query/etag", test_query_etag);
+	g_test_add_data_func ("/calendar/acls/get_rules", service, test_acls_get_rules);
 	if (g_test_slow () == TRUE) {
-		g_test_add_func ("/calendar/acls/insert_rule", test_acls_insert_rule);
-		g_test_add_func ("/calendar/acls/update_rule", test_acls_update_rule);
-		g_test_add_func ("/calendar/acls/delete_rule", test_acls_delete_rule);
+		g_test_add_data_func ("/calendar/acls/insert_rule", service, test_acls_insert_rule);
+		g_test_add_data_func ("/calendar/acls/update_rule", service, test_acls_update_rule);
+		g_test_add_data_func ("/calendar/acls/delete_rule", service, test_acls_delete_rule);
 	}
 
 	retval = g_test_run ();
-	if (service != NULL)
-		g_object_unref (service);
+	g_object_unref (service);
 
 	return retval;
 }
