@@ -33,7 +33,7 @@
 
 #define PW_USERNAME "libgdata.picasaweb@gmail.com"
 /* the following two properties will change if a new album is added */
-#define NUM_ALBUMS 3
+#define NUM_ALBUMS 5
 #define TEST_ALBUM_INDEX 2
 
 static void
@@ -62,6 +62,7 @@ delete_directory (GFile *directory, GError **error)
 	}
 
 	g_file_enumerator_close (enumerator, NULL, error);
+	g_object_unref (enumerator);
 
 	/* Delete the directory itself */
 	g_file_delete (directory, NULL, error);
@@ -171,15 +172,15 @@ test_upload_async (gconstpointer service)
 	time_str = g_time_val_to_iso8601 (&timeval);
 	summary = g_strdup_printf ("Async Photo Summary (%s)", time_str);
 
-	expected_xml = g_strdup_printf ("<entry "
-						"xmlns='http://www.w3.org/2005/Atom' "
-						"xmlns:gphoto='http://schemas.google.com/photos/2007' "
-						"xmlns:media='http://search.yahoo.com/mrss/' "
-						"xmlns:gd='http://schemas.google.com/g/2005' "
-						"xmlns:exif='http://schemas.google.com/photos/exif/2007' "
-						"xmlns:app='http://www.w3.org/2007/app' "
-						"xmlns:georss='http://www.georss.org/georss' "
-						"xmlns:gml='http://www.opengis.net/gml'>"
+	expected_xml = g_strdup_printf ("<entry ("
+						"xmlns='http://www.w3.org/2005/Atom' ?|"
+						"xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
+						"xmlns:media='http://search.yahoo.com/mrss/' ?|"
+						"xmlns:gd='http://schemas.google.com/g/2005' ?|"
+						"xmlns:exif='http://schemas.google.com/photos/exif/2007' ?|"
+						"xmlns:app='http://www.w3.org/2007/app' ?|"
+						"xmlns:georss='http://www.georss.org/georss' ?|"
+						"xmlns:gml='http://www.opengis.net/gml' ?){8}>"
 						"<title type='text'>Async Photo Entry Title</title>"
 						"<summary type='text'>Async Photo Summary \\(%s\\)</summary>"
 						"<gphoto:position>0</gphoto:position>"
@@ -206,7 +207,7 @@ test_upload_async (gconstpointer service)
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 1);
+	parsed_time_str = g_match_info_fetch (match_info, 2);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
 	g_assert_cmpuint (abs (delta), <, 100);
 
@@ -539,15 +540,15 @@ test_upload_simple (gconstpointer service)
 	time_str = g_time_val_to_iso8601 (&timeval);
 	summary = g_strdup_printf ("Photo Summary (%s)", time_str);
 
-	expected_xml = g_strdup_printf ("<entry "
-						"xmlns='http://www.w3.org/2005/Atom' "
-						"xmlns:gphoto='http://schemas.google.com/photos/2007' "
-						"xmlns:media='http://search.yahoo.com/mrss/' "
-						"xmlns:gd='http://schemas.google.com/g/2005' "
-						"xmlns:exif='http://schemas.google.com/photos/exif/2007' "
-						"xmlns:app='http://www.w3.org/2007/app' "
-						"xmlns:georss='http://www.georss.org/georss' "
-						"xmlns:gml='http://www.opengis.net/gml'>"
+	expected_xml = g_strdup_printf ("<entry ("
+						"xmlns='http://www.w3.org/2005/Atom' ?|"
+						"xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
+						"xmlns:media='http://search.yahoo.com/mrss/' ?|"
+						"xmlns:gd='http://schemas.google.com/g/2005' ?|"
+						"xmlns:exif='http://schemas.google.com/photos/exif/2007' ?|"
+						"xmlns:app='http://www.w3.org/2007/app' ?|"
+						"xmlns:georss='http://www.georss.org/georss' ?|"
+						"xmlns:gml='http://www.opengis.net/gml' ?){8}>"
 						"<title type='text'>Photo Entry Title</title>"
 						"<summary type='text'>Photo Summary \\(%s\\)</summary>"
 						"<gphoto:position>0</gphoto:position>"
@@ -574,7 +575,7 @@ test_upload_simple (gconstpointer service)
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 1);
+	parsed_time_str = g_match_info_fetch (match_info, 2);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
 	g_assert_cmpuint (abs (delta), <, 100);
 
@@ -738,6 +739,8 @@ test_photo (gconstpointer service)
 	g_assert_cmpint (gdata_media_thumbnail_get_time (thumbnail), ==, -1); /* PicasaWeb doesn't set anything better */
 
 	g_free (timestamp);
+	g_object_unref (album_feed);
+	g_object_unref (photo_feed);
 }
 
 static void
@@ -750,6 +753,7 @@ test_photo_feed_entry (gconstpointer service)
 	GDataPicasaWebAlbum *album;
 	GList *albums;
 	GList *files;
+	gchar *xml;
 	GDataEntry *photo_entry;
 	gchar *str;
 	GTimeVal _time;
@@ -793,8 +797,14 @@ test_photo_feed_entry (gconstpointer service)
 
 	g_assert_cmpstr (gdata_entry_get_content (photo_entry), ==,
 			 "http://lh3.ggpht.com/_1kdcGyvOb8c/SfQFWPnuovI/AAAAAAAAAB0/MI0L4Sd11Eg/100_0269.jpg");
-	g_assert_cmpstr (gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry)), !=, NULL);
-	g_assert_cmpuint (strlen (gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry))), >, 0);
+
+	xml = gdata_parsable_get_xml (GDATA_PARSABLE (photo_entry));
+	g_assert_cmpstr (xml, !=, NULL);
+	g_assert_cmpuint (strlen (xml), >, 0);
+	g_free (xml);
+
+	g_object_unref (album_feed);
+	g_object_unref (photo_feed);
 }
 
 static void
@@ -831,6 +841,9 @@ test_photo_feed (gconstpointer service)
 	g_assert_cmpuint (gdata_feed_get_items_per_page (photo_feed), ==, 1000);
 	g_assert_cmpuint (gdata_feed_get_start_index (photo_feed), ==, 1);
 	g_assert_cmpuint (gdata_feed_get_total_results (photo_feed), ==, 1);
+
+	g_object_unref (album_feed);
+	g_object_unref (photo_feed);
 }
 
 static void
@@ -1025,10 +1038,12 @@ test_album_feed (gconstpointer service)
 	/* TODO find out why subtitle == null when returned: no subtitle for feed? printf("feed subtitle: %s\n", gdata_feed_get_subtitle(feed)); */
 	g_assert_cmpstr (gdata_feed_get_id (album_feed), ==, "http://picasaweb.google.com/data/feed/user/libgdata.picasaweb");
 	g_assert_cmpstr (gdata_feed_get_etag (album_feed), !=, NULL); /* this varies as albums change, like when a new image is uploaded in our test! */
-	g_assert_cmpstr (gdata_feed_get_icon (album_feed), ==, "http://lh6.ggpht.com/_1kdcGyvOb8c/AAAA9mDag3s/AAAAAAAAAAA/Jq-NWYWKFao/s64-c/libgdata.picasaweb.jpg");
+	g_assert_cmpstr (gdata_feed_get_icon (album_feed), ==, "http://lh3.ggpht.com/_1kdcGyvOb8c/AAAAAAAAAAA/AAAAAAAAAAA/jNhBfdaNdD4/s64-c/libgdata.picasaweb.jpg");
 	g_assert_cmpuint (gdata_feed_get_items_per_page (album_feed), ==, 1000);
 	g_assert_cmpuint (gdata_feed_get_start_index (album_feed), ==, 1);
 	g_assert_cmpuint (gdata_feed_get_total_results (album_feed), ==, NUM_ALBUMS);
+
+	g_object_unref (album_feed);
 }
 
 static void
@@ -1109,6 +1124,7 @@ test_query_all_albums (gconstpointer service)
 	g_assert_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER);
 	g_assert (album_feed == NULL);
 	g_clear_error (&error);
+	g_object_unref (query);
 
 	/* Now try a proper query */
 	album_feed = gdata_picasaweb_service_query_all_albums (GDATA_PICASAWEB_SERVICE (service), NULL, NULL, NULL, NULL, NULL, &error);
@@ -1146,7 +1162,7 @@ test_query_user (gconstpointer service)
 	g_assert_cmpint (gdata_picasaweb_user_get_quota_limit (user), ==, 1073741824); /* 1GiB: it'll be a beautiful day when this assert gets tripped */
 	g_assert_cmpint (gdata_picasaweb_user_get_quota_current (user), >, 0);
 	g_assert_cmpint (gdata_picasaweb_user_get_max_photos_per_album (user), >, 0); /* now it's 1000, testing this weakly to avoid having to regularly update it */
-	g_assert_cmpstr (gdata_picasaweb_user_get_thumbnail_uri (user), ==, "http://lh6.ggpht.com/_1kdcGyvOb8c/AAAA9mDag3s/AAAAAAAAAAA/Jq-NWYWKFao/s64-c/libgdata.picasaweb.jpg");
+	g_assert_cmpstr (gdata_picasaweb_user_get_thumbnail_uri (user), ==, "http://lh3.ggpht.com/_1kdcGyvOb8c/AAAAAAAAAAA/AAAAAAAAAAA/jNhBfdaNdD4/s64-c/libgdata.picasaweb.jpg");
 
 	g_object_unref (user);
 }
@@ -1250,14 +1266,15 @@ test_album_new (gconstpointer service)
 	/* Get the current time */
 	g_get_current_time (&timeval);
 
-	/* Build a regex to match the timestamp from the XML, since we can't definitely say what it'll be */
-	regex = g_regex_new ("<entry xmlns='http://www.w3.org/2005/Atom' "
-				    "xmlns:gphoto='http://schemas.google.com/photos/2007' "
-				    "xmlns:media='http://search.yahoo.com/mrss/' "
-				    "xmlns:gd='http://schemas.google.com/g/2005' "
-				    "xmlns:gml='http://www.opengis.net/gml' "
-				    "xmlns:app='http://www.w3.org/2007/app' "
-				    "xmlns:georss='http://www.georss.org/georss'>"
+	/* Build a regex to match the timestamp from the XML, since we can't definitely say what it'll be. Note that we also assign any order to the
+	 * namespace definitions, since due to a change in GLib's hashing algorithm, they could be in different orders with different GLib versions. */
+	regex = g_regex_new ("<entry (xmlns='http://www.w3.org/2005/Atom' ?|"
+				     "xmlns:gphoto='http://schemas.google.com/photos/2007' ?|"
+				     "xmlns:media='http://search.yahoo.com/mrss/' ?|"
+				     "xmlns:gd='http://schemas.google.com/g/2005' ?|"
+				     "xmlns:gml='http://www.opengis.net/gml' ?|"
+				     "xmlns:app='http://www.w3.org/2007/app' ?|"
+				     "xmlns:georss='http://www.georss.org/georss' ?){7}>"
 					"<title type='text'></title>"
 					"<id>http://picasaweb.google.com/data/entry/user/libgdata.picasaweb/albumid/5328889949261497249</id>"
 					"<gphoto:id>5328889949261497249</gphoto:id>"
@@ -1276,7 +1293,7 @@ test_album_new (gconstpointer service)
 	 * for the photo. */
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (album));
 	g_assert (g_regex_match (regex, xml, 0, &match_info) == TRUE);
-	parsed_time_str = g_match_info_fetch (match_info, 1);
+	parsed_time_str = g_match_info_fetch (match_info, 2);
 	delta = g_ascii_strtoull (parsed_time_str, NULL, 10) - (((guint64) timeval.tv_sec) * 1000 + ((guint64) timeval.tv_usec) / 1000);
 	g_assert_cmpuint (abs (delta), <, 100);
 
