@@ -267,7 +267,6 @@ test_upload_simple (gconstpointer service)
 	GDataYouTubeVideo *video, *new_video;
 	GDataMediaCategory *category;
 	GFile *video_file;
-	gchar *xml;
 	GError *error = NULL;
 
 	video = gdata_youtube_video_new (NULL);
@@ -280,8 +279,7 @@ test_upload_simple (gconstpointer service)
 	gdata_youtube_video_set_keywords (video, "toast, wedding");
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (video,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -298,7 +296,6 @@ test_upload_simple (gconstpointer service)
 					"<app:draft>no</app:draft>"
 				"</app:control>"
 			 "</entry>");
-	g_free (xml);
 
 	/* TODO: fix the path */
 	video_file = g_file_new_for_path (TEST_FILE_DIR "sample.ogg");
@@ -373,7 +370,6 @@ test_parsing_yt_recorded (gconstpointer service)
 {
 	GDataYouTubeVideo *video;
 	GTimeVal recorded;
-	gchar *xml;
 	GError *error = NULL;
 
 	video = GDATA_YOUTUBE_VIDEO (gdata_parsable_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO,
@@ -414,8 +410,7 @@ test_parsing_yt_recorded (gconstpointer service)
 	gdata_youtube_video_set_recorded (video, &recorded);
 
 	/* Check the XML */
-	xml = gdata_parsable_get_xml (GDATA_PARSABLE (video));
-	g_assert_cmpstr (xml, ==,
+	gdata_test_assert_xml (video,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
 				"xmlns:media='http://search.yahoo.com/mrss/' "
 				"xmlns:gd='http://schemas.google.com/g/2005' "
@@ -442,7 +437,6 @@ test_parsing_yt_recorded (gconstpointer service)
 					"<app:draft>no</app:draft>"
 				"</app:control>"
 			 "</entry>");
-	g_free (xml);
 
 	/* TODO: more tests on entry properties */
 
